"""
This is a setup.py script generated by py2applet

Usage:
    python setup.py py2app
"""
from Cython.Build import cythonize
from setuptools import setup

APP = ['main.py']
DATA_FILES = []
OPTIONS = {}

setup(
    app=APP,
    data_files=DATA_FILES,
    options={'py2app': OPTIONS},
    ext_modules=cythonize("alacorder4.py","alacorder4c.py"),
    setup_requires=['py2app'],
)
